@@ -7,12 +7,12 @@ from math import ceil, floor, log10
 from trytond.config import config
 from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, DeactivableMixin, fields, Check, SymbolMixin)
-from trytond.model.exceptions import AccessError
+    Check, DeactivableMixin, DigitsMixin, ModelSQL, ModelView, SymbolMixin,
+    fields)
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
 
-from .exceptions import UOMValidationError
+from .exceptions import UOMAccessError, UOMValidationError
 
 __all__ = ['uom_conversion_digits']
 
@@ -32,7 +32,7 @@ class UomCategory(ModelSQL, ModelView):
         cls._order.insert(0, ('name', 'ASC'))
 
 
-class Uom(SymbolMixin, DeactivableMixin, ModelSQL, ModelView):
+class Uom(SymbolMixin, DigitsMixin, DeactivableMixin, ModelSQL, ModelView):
     "Unit of Measure"
     __name__ = 'product.uom'
     name = fields.Char("Name", size=None, required=True, translate=True)
@@ -57,7 +57,6 @@ class Uom(SymbolMixin, DeactivableMixin, ModelSQL, ModelView):
         domain=[
             ('rounding', '>', 0),
             ],
-        depends=['digits'],
         help="The accuracy to which values are rounded.")
     digits = fields.Integer(
         "Display Digits", required=True,
@@ -75,7 +74,7 @@ class Uom(SymbolMixin, DeactivableMixin, ModelSQL, ModelView):
 
     @classmethod
     def check_xml_record(cls, records, values):
-        return True
+        pass
 
     @staticmethod
     def default_rate():
@@ -129,22 +128,25 @@ class Uom(SymbolMixin, DeactivableMixin, ModelSQL, ModelView):
         return _round(self, number, func=floor)
 
     @classmethod
-    def validate(cls, uoms):
-        super(Uom, cls).validate(uoms)
-        for uom in uoms:
-            uom.check_factor_and_rate()
+    def validate_fields(cls, uoms, field_names):
+        super().validate_fields(uoms, field_names)
+        cls.check_factor_and_rate(uoms, field_names)
 
-    def check_factor_and_rate(self):
+    @classmethod
+    def check_factor_and_rate(cls, uoms, field_names=None):
         "Check coherence between factor and rate"
-        if self.rate == self.factor == 0.0:
+        if field_names and not (field_names & {'rate', 'factor'}):
             return
-        if (self.rate != round(
-                    1.0 / self.factor, uom_conversion_digits[1])
-                and self.factor != round(
-                    1.0 / self.rate, uom_conversion_digits[1])):
-            raise UOMValidationError(
-                gettext('product.msg_uom_incompatible_factor_rate',
-                    uom=self.rec_name))
+        for uom in uoms:
+            if uom.rate == uom.factor == 0.0:
+                continue
+            if (uom.rate != round(
+                        1.0 / uom.factor, uom_conversion_digits[1])
+                    and uom.factor != round(
+                        1.0 / uom.rate, uom_conversion_digits[1])):
+                raise UOMValidationError(
+                    gettext('product.msg_uom_incompatible_factor_rate',
+                        uom=uom.rec_name))
 
     @classmethod
     def write(cls, *args):
@@ -152,26 +154,25 @@ class Uom(SymbolMixin, DeactivableMixin, ModelSQL, ModelView):
             super(Uom, cls).write(*args)
             return
 
-        actions = iter(args)
-        all_uoms = []
-        for uoms, values in zip(actions, actions):
-            if 'rate' not in values and 'factor' not in values \
-                    and 'category' not in values:
-                continue
-            all_uoms += uoms
-
-        old_uom = dict((uom.id, (uom.factor, uom.rate, uom.category))
-            for uom in all_uoms)
+        all_uoms = sum(args[0:None:2], [])
+        old_uom = {
+            uom.id: (uom.factor, uom.rate, uom.category) for uom in all_uoms}
+        old_digits = {uom.id: uom.digits for uom in all_uoms}
 
         super(Uom, cls).write(*args)
 
         for uom in all_uoms:
             for i, field in enumerate(['factor', 'rate', 'category']):
                 if getattr(uom, field) != old_uom[uom.id][i]:
-                    raise AccessError(
+                    raise UOMAccessError(
                         gettext('product.msg_uom_modify_%s' % field,
                             uom=uom.rec_name),
                         gettext('product.msg_uom_modify_options'))
+            if uom.digits < old_digits[uom.id]:
+                raise UOMAccessError(
+                    gettext('product.msg_uom_decrease_digits',
+                        uom=uom.rec_name),
+                    gettext('product.msg_uom_modify_options'))
 
     @property
     def accurate_field(self):
@@ -194,9 +195,9 @@ class Uom(SymbolMixin, DeactivableMixin, ModelSQL, ModelView):
         if not qty or (from_uom is None and to_uom is None):
             return qty
         if from_uom is None:
-            raise ValueError("missing from_uom")
+            raise ValueError("missing from_UoM")
         if to_uom is None:
-            raise ValueError("missing to_uom")
+            raise ValueError("missing to_UoM")
         if from_uom.category.id != to_uom.category.id:
             if not factor and not rate:
                 raise ValueError(
@@ -242,9 +243,9 @@ class Uom(SymbolMixin, DeactivableMixin, ModelSQL, ModelView):
         if not price or (from_uom is None and to_uom is None):
             return price
         if from_uom is None:
-            raise ValueError("missing from_uom")
+            raise ValueError("missing from_UoM")
         if to_uom is None:
-            raise ValueError("missing to_uom")
+            raise ValueError("missing to_UoM")
         if from_uom.category.id != to_uom.category.id:
             if not factor and not rate:
                 raise ValueError(
